#
# Tilt-Platform Control Program to balance a ball on the platform
#
#
# Original Program from Johan Link:
# https://github.com/JohanLink/Ball-Balancing-PID-System
# modified and adapted to (Fischertechni)k-Tilt-Platform
# by   Florian Bauer June 2023
#
# uses a USB camera for ball tracking
#  camera must be installed over the platform such that the
#  - camera views the platform with ball and platform should fill the image
#  - actuators of platform are on top side of image
#
# uses Arduino as Servo Controller
#  communication: Serial via USB
#  if no connection to Arduino is found see function connectArduino() in this program  for further information!
#
#
#  Serial Communication Protocol:
#  <x>,<y>   tilt platform to x- and y-angle, range -20..20
#
#  relevant if a DC-Motor is used as Servo
#  kp<p>     set kp of Servo-PID to p
#  ki<i>     set ki of Servo-PID to i
#  kd<d>     set kd of Servo-PID to d
#  l<l>      set limit of Servo-PID to l
#  r         set actual position as zero position, reset encoder counters
#  p         toggle print mode : if on print all parameters
#  c         toggle control mode : if on Joystick-Control of platform
#            Analog-Joystick connected to analog pins A2 and A3
#
#  This control program calculates the the actuator angles by: 
#    alpha = Iy * IyToAlpha
#    beta = Ix *  IxToAlpha
#
#  these constants and their signs can be adjusted in the top of the code
#  section 
#
#
#  Program Usage:
#   start with
#     python interface.py
#
#   Control UI:
#   Step 1: Start Video Feedback and Lock ball
#     Show Video Feedback - opens Camera view with Ball
#        Middle Mouse Click on Ball to lock ball color for tracking
#        eventually adjust sensibility silder
#        when the ball is detected it shows a yellow ring
#        Left click to spot target position on platform (green ring)
#   Step 2: Set Platform zo Zero position which should be horizontal
#     Servo control
#        Setup platform - Platform moves to 0,0
#     If the platform is not horizontal, adjust mechanically or use
#     Platform Control
#        In Stop mode: X and Y angles can be tuned
#        Set Zero will set this position as zero position in the Arduino controller 
#    Step 3:
#       Ball Control
#         Click To Start up to start balancing
#
#    Step 4:
#        Adjust PID-coefficients for optimal tracking
#
#    Step 5 (optional):
#      Servo control:
#        Parameters of Servo controller in Arduino can be adjusted
#
#    Step 6: 
#       Click Stop to stop balancing
#
#
# Trouble shooting:
#
#   Hint: Start with a light ball (low inertia) first
#
#   Arduino not connected / not found:
#     - observe output of program at start and eventually modify 
#       the identification code in connectArduino() in this program 
#       with the "right" identification string
#
#   ball tracking does not work properly 
#     - middle-click on ball
#     - adjust video parameters
#     - adjust PID parameters
#     - patform not leveled correctly
#     - misalignment of actuators
#     - Servo-PID parameters not optimal
#
#   ball kicks wildly around
#     - Target position got lost => Left click to target position
#     - D-Value too high (set to 1-2)
#     - P-Value too high 
#     - proportianal values IxToAlpha or IyToAlpha for angle calculation too high
#   ball oscillates around target position
#     - I-value too high
#     - P,D-values not correct
#
#   platform is tilting always in wrong direction to balance ball
#      - Left and right servo motors are interchanged
#      - camera is rotated
#      - platform zero position not/no longer horizontally


import cv2
import numpy as np
import time
import imutils
import tkinter as tk
import tkinter.messagebox
from PIL import Image, ImageTk
#import serial
import serial.tools.list_ports
from math import *

# proportional values for controller angle calculation:
IxToAlpha = 20
IyToAlpha = 20
IxSgn =  1
IySgn =  1
IswapXY   =  False 

# the most important function in this program:
def donothing():
    pass

def setConsigneWithMouse(mousePosition):
#     PID coefficients
#      Adjust D to minimum
#
#     Ball Control
#        To Start up - controller starts balancing
#        Stop        - controller stops balancing
#
    global consigneX, consigneY
    if mousePosition.y > 10:
        refreshGraph()
        consigneX,consigneY = mousePosition.x,mousePosition.y


def getMouseClickPosition(mousePosition):
    global mouseX,mouseY
    global getPixelColor
    mouseX,mouseY = mousePosition.x,mousePosition.y
    getPixelColor = True

lines = open("data.txt").read().splitlines()
lines = lines[:-11]     #enlève les 11 dernières lignes du fichier
lines = lines[1:]       #enlève la première ligne du fichier

dataDict = {}

camHeight = 480
camWidth = 640
cam = cv2.VideoCapture(0)
cam.set(3,camWidth)
cam.set(4,camHeight)

getPixelColor = False
H,S,V = 0,0,0

mouseX,mouseY = 0,0

for i in range(0,len(lines)):
    key, value = lines[i].split("#")
    alpha, beta = key.split("|")
    angleA, angleB, angleC = value.split("|")
    dataDict[(float(alpha),float(beta))] = (float(angleA), float(angleB), float(angleC))

controllerWindow = tk.Tk()
#controllerWindow.title("fenêtre de contrôle")
controllerWindow.title("Control Window")
#controllerWindow.geometry("820x500")
controllerWindow.geometry("820x770")
controllerWindow["bg"]="white"
controllerWindow.resizable(0, 0)

videoWindow = tk.Toplevel(controllerWindow)
videoWindow.title("Caméra")
videoWindow.resizable(0, 0)  #empeche de modifier les dimensions de la fenêtre

videoWindow.protocol("WM_DELETE_WINDOW",donothing)
videoWindow.bind("<Button-2>",getMouseClickPosition)
videoWindow.bind("<Button-1>",setConsigneWithMouse)

lmain = tk.Label(videoWindow)
lmain.pack()
videoWindow.withdraw()

graphWindow = tk.Toplevel(controllerWindow)
#graphWindow.title("Position en fonction du temps")
graphWindow.title("Position vs. time ")
graphWindow.resizable(0, 0)
graphCanvas = tk.Canvas(graphWindow,width=camHeight+210,height=camHeight)
graphCanvas.pack()
graphWindow.withdraw()

pointsListCircle = []
def createPointsListCircle(rayon):
    global pointsListCircle
    for angle in range(0,360):
        angle=angle-90
        pointsListCircle.append([rayon*cos(radians(angle))+240,rayon*sin(radians(angle))+240])
#createPointsListCircle(150)
createPointsListCircle(60)

pointsListEight = []
def createPointsListEight(rayon):
    global pointsListEight
    for angle in range(270,270+360):
        pointsListEight.append([rayon*cos(radians(angle))+240,rayon*sin(radians(angle))+240+rayon])
    for angle in range(360,0,-1):
        angle=angle+90
        pointsListEight.append([rayon*cos(radians(angle))+240,rayon*sin(radians(angle))+240-rayon])
#createPointsListEight(80)
createPointsListEight(60)

drawCircleBool = False
def startDrawCircle():
    global drawCircleBool, drawEightBool, consigneX, consigneY
    if drawCircleBool == False:
        drawCircleBool = True
        #BballDrawCircle["text"] = "Centrer la bille"
        BballDrawCircle["text"] = "Center ball"
    else:
        drawCircleBool = False
        consigneX, consigneY =  240, 240
        sliderCoefP.set(sliderCoefPDefault)
        #BballDrawCircle["text"] = "Faire tourner la bille en cercle"
        BballDrawCircle["text"] = "Spin the ball in a circle"

drawEightBool = False
def startDrawEight():
    global drawEightBool, drawCircleBool, consigneX, consigneY
    if drawEightBool == False:
        drawEightBool = True
        #BballDrawEight["text"] = "Centrer la bille"
        BballDrawCircle["text"] = "Center ball"
    else:
        drawEightBool = False
        consigneX, consigneY =  240, 240
        sliderCoefP.set(sliderCoefPDefault)
#        BballDrawEight["text"] = "Faire tourner la bille en huit"
        BballDrawEight["text"] = "Spin the ball in a figure eight"

pointCounter = 0
def drawWithBall():
    global pointCounter, consigneX, consigneY
    if drawCircleBool == True:
        sliderCoefP.set(15)
        if pointCounter >= len(pointsListCircle):
            pointCounter = 0
        point = pointsListCircle[pointCounter]
        consigneX, consigneY = point[0], point[1]
        pointCounter += 7
    if drawEightBool == True:
        sliderCoefP.set(15)
        if pointCounter >= len(pointsListEight):
            pointCounter = 0
        point = pointsListEight[pointCounter]
        consigneX, consigneY = point[0], point[1]
        pointCounter += 7


showVideoWindow = False
def showCameraFrameWindow():
    global showVideoWindow, showGraph
    global BRetourVideoTxt
    if showVideoWindow == False:
        if showGraph == True:
            graphWindow.withdraw()
            showGraph = False
            #BafficherGraph["text"] = "Afficher graphique"
            BafficherGraph["text"] = "Show graph"
        videoWindow.deiconify()
        showVideoWindow = True
        #BRetourVideo["text"] = "Cacher le retour vidéo "
        BRetourVideo["text"] = "Close Video"
    else:
        videoWindow.withdraw()
        showVideoWindow = False
        #BRetourVideo["text"] = "Afficher le retour vidéo"
        BRetourVideo["text"] = "Open Video"

showCalqueCalibrationBool = False
def showCalqueCalibration():
    global showCalqueCalibrationBool
    showCalqueCalibrationBool = not showCalqueCalibrationBool

showGraph = False
def showGraphWindow():
    global showGraph, showVideoWindow
    global BafficherGraph
    
    if showGraph == False:
        if showVideoWindow == True:
            videoWindow.withdraw()
            showVideoWindow = False
            #BRetourVideo["text"] = "Afficher le retour vidéo"
            BRetourVideo["text"] = "Open Video"
        showGraph = True
        #BafficherGraph["text"] = "Cacher graphique "
        BafficherGraph["text"] = "Close graph"
    else:
        showGraph = False
        #BafficherGraph["text"] = "Afficher graphique"
        BafficherGraph["text"] = "Open graph"

t = 480
consigneY = 240
consigneX = 240
def paintGraph():
    global t,consigneY,x,y,prevX,prevY,alpha,prevAlpha
    global showGraphPositionX,showGraphPositionY, showGraphAlpha
    if showGraph == True:
        graphWindow.deiconify()
        if showGraphPositionX.get() == 1:
            graphCanvas.create_line(t-3,prevX,t,x, fill="#b20000", width=2)
        if showGraphPositionY.get() == 1:
            graphCanvas.create_line(t-3,prevY,t,y, fill="#0069b5", width=2)
        if showGraphAlpha.get() == 1:
            graphCanvas.create_line(t-3,240-prevAlpha*3,t,240-alpha*3, fill="#8f0caf", width=2)
        if t >= 480:
            t = 0
            graphCanvas.delete("all")
            graphCanvas.create_line(3,3,480,3,fill="black", width=3)
            graphCanvas.create_line(3,480,480,480,fill="black", width=3)
            graphCanvas.create_line(3,3,3,480,fill="black", width=3)
            graphCanvas.create_line(480,3,480,480,fill="black", width=3)
            graphCanvas.create_line(550,32,740,32,fill="#b20000", width=5)
            graphCanvas.create_line(550,53,740,53,fill="#0069b5", width=5)
            graphCanvas.create_line(550,73,740,73,fill="#8f0caf", width=5)
            if showGraphPositionX.get() == 1:
                graphCanvas.create_line(3,consigneX,480,consigneX,fill="#ff7777", width=2)
            if showGraphPositionY.get() == 1:
                graphCanvas.create_line(3,consigneY,480,consigneY,fill="#6f91f7", width=2)
        t += 3
    else:
        graphWindow.withdraw()


def refreshGraph():
    global t
    t=480

def endProgam():
    controllerWindow.destroy()


sliderHDefault = 15
sliderSDefault = 70
sliderVDefault = 70
sliderCoefPDefault = 6
sliderCoefIDefault = 0.1
sliderCoefDDefault = 1.5

def resetSlider():
    sliderH.set(sliderHDefault)
    sliderS.set(sliderSDefault)
    sliderV.set(sliderVDefault)
    sliderCoefP.set(sliderCoefPDefault)
    sliderCoefI.set(sliderCoefIDefault)
    sliderCoefD.set(sliderCoefDDefault)


def rangerPlateau():
    if arduinoIsConnected == True:
        #if tkinter.messagebox.askokcancel("Avertissement", "Pensez à retirer le plateau."):
        if tkinter.messagebox.askokcancel("Warning", "Pensez à retirer le plateau."):
            #print("abaissement des bras")
            print("lowering arms")
#            ser.write(("descendreBras\n").encode())
    else:
        #if tkinter.messagebox.askokcancel("Avertissement","L'Arduino n'est pas connecté"):
        if tkinter.messagebox.askokcancel("Warning","Arduino not connected"):
            donothing()


def eleverPlateau():
    global alpha
    if arduinoIsConnected == True:
        #if tkinter.messagebox.askokcancel("Avertissement", "Pensez à retirer le plateau."):
        if tkinter.messagebox.askokcancel("Warning", "Pensez à retirer le plateau."):
            #print("Elevation des bras")
            print("Lift arms")
            #ser.write((str(dataDict[(0,0)])+"\n").encode())
            ser.write("0,0\n".encode())
#            print("eleverPlateau:",(str(dataDict[(0,0)])+"\n").encode())

            alpha = 0
    else:
        #if tkinter.messagebox.askokcancel("Avertissement","L'Arduino n'est pas connecté"):
        if tkinter.messagebox.askokcancel("Warning","Arduino not connected"):
            donothing()

def servoTestAxis(axis):
    print("Test Servo Axis ", axis)

    if arduinoIsConnected == True:
        #if tkinter.messagebox.askokcancel("Avertissement", "Le plateau doit être en place."):
        if tkinter.messagebox.askokcancel("Warning", "The platform must be in place.\nTesting Axis {}".format(axis)):
            #w0=35
            w0=10
            #for i in range(10):
            for i in range(3):
                w= i*5
                if axis==0:
                    ser.write(("{},{}\n".format(w,w0)).encode())
                if axis==1:
                    ser.write(("{},{}\n".format(w0,w)).encode())
                #if axis==0:
                #    ser.write(("{},{},{}\n".format(w,w0,w0)).encode())
                #if axis==1:
                #    ser.write(("{},{},{}\n".format(w0,w,w0)).encode())
                #if axis==2:
                #    ser.write(("{},{},{}\n".format(w0,w0,w)).encode())
                ser.flush()
                time.sleep(0.2)
            time.sleep(1)
            #ser.write((str(dataDict[(0,0)])+"\n").encode())
            ser.write(("0,0\n").encode())
    else:
        #if tkinter.messagebox.askokcancel("Avertissement","L'Arduino n'est pas connecté"):
        if tkinter.messagebox.askokcancel("Warning","Arduino not connected"):
            donothing()

def servoTestAxis_0():
    servoTestAxis(0)
def servoTestAxis_1():
    servoTestAxis(1)
def servoTestAxis_2():
    servoTestAxis(2)

def servosTest():
    if arduinoIsConnected == True:
        #if tkinter.messagebox.askokcancel("Avertissement", "Le plateau doit être en place."):
        if tkinter.messagebox.askokcancel("Warning", "The platform must be in place."):
            for i in range(2):
                beta = -20
                #alpha = 35
                alpha = 0
                while beta < 20:
#                    ser.write((str(dataDict[(alpha,beta)])+"\n").encode())
                    print(alpha,beta)
#                    ser.write((str(alpha) + "," + str(beta)+"\n").encode())
                    ser.flush()
                    time.sleep(0.002)
                    beta = round(beta+0.2,2)
                    print(alpha,beta)
            time.sleep(1)
#            ser.write((str(dataDict[(0,0)])+"\n").encode())
#            ser.write(("0,0\n").encode())
    else:
        #if tkinter.messagebox.askokcancel("Avertissement","L'Arduino n'est pas connecté"):
        if tkinter.messagebox.askokcancel("Warning","Arduino not connected"):
            donothing()


arduinoIsConnected = False
def connectArduino():
    global ser
    global label
    global arduinoIsConnected
    ports = list(serial.tools.list_ports.comports())
    for p in ports:
        print("port:",p.description)
        #
        # IMPORTANT: 
        #  if no connection is established it might be necessary to add
        #  another comparison string:

        if "ttyACM0" in p.description or "USB2.0-Serial" in p.description:
            print("Arduino:",p)
            ser = serial.Serial(p[0], 250000, timeout=1)
            time.sleep(1) #give the connection a second to settle
            #label.configure(text="Arduino connecté", fg="#36db8b")
            label.configure(text="Arduino connected", fg="#36db8b")
            arduinoIsConnected = True

startBalanceBall = False
def startBalance():
    global startBalanceBall
    if arduinoIsConnected == True:
        if startBalanceBall == False:
            startBalanceBall = True
            #BStartBalance["text"] = "Arrêter"
            BStartBalance["text"] = "Stop"
        else:
            startBalanceBall = False
            #BStartBalance["text"] = "Commencer"
            BStartBalance["text"] = "Start"
    else:
        #if tkinter.messagebox.askokcancel("Avertissement","L'Arduino n'est pas connecté"):
        if tkinter.messagebox.askokcancel("Warning","Arduino not connected"):
            donothing()

sommeErreurX = 1
sommeErreurY = 1
timeInterval = 1
alpha, beta, prevAlpha, prevBeta = 0,0,0,0
omega = 0.2
def PIDcontrol(ballPosX, ballPosY, prevBallPosX, prevBallPosY, consigneX, consigneY):
    global omega
    global sommeErreurX, sommeErreurY
    global alpha, beta, prevAlpha, prevBeta
    global startBalanceBall, arduinoIsConnected
    
    Kp = sliderCoefP.get()
    Ki = sliderCoefI.get()
    Kd = sliderCoefD.get()

    Ix = Kp*(consigneX-ballPosX) + Ki*sommeErreurX + Kd*((prevBallPosX-ballPosX)/0.0333)
    Iy = Kp*(consigneY-ballPosY) + Ki*sommeErreurY + Kd*((prevBallPosY-ballPosY)/0.0333)
    
    
    Ix = round(Ix/10000, 5)
    Iy = round(Iy/10000, 5)
    

    #Ax =consigneX-ballPosX
    #Ay =consigneY-ballPosY
    #A= sqrt(Ax**2 + Ay**2)
    #if A != 0:
    #  alpha= degrees(asin(Ax/A))
    #  beta = degrees(asin(Ay/A))

    #elif Ix != 0 and sqrt(Ix**2 + Iy**2) < 1:
    #    beta = atan(Iy/Ix)
    #    alpha = asin(sqrt(Ix**2 + Iy**2))
    #    beta = degrees(beta)
    #    alpha = degrees(alpha)
    #    if Ix < 0 and Iy >= 0:
    #        beta = abs(beta)
    #    elif Ix > 0 and Iy >= 0:
    #        beta = 180-abs(beta)
    #    elif Ix > 0 and Iy <= 0:
    #        beta = 180+abs(beta)
    #    elif Ix < 0 and Iy <= 0:
    #        beta = 360-abs(beta)
#
    #elif Ix == 0 and sqrt(Ix**2 + Iy**2) < 1:
    #    if Iy > 0:
    #        beta = 0
    #        alpha = asin(sqrt(Ix**2 + Iy**2))
    #    elif Iy < 0:
    #        beta = 0
    #        alpha = asin(sqrt(Ix**2 + Iy**2))
    #    alpha = degrees(alpha)
##
#    elif Ix != 0 and sqrt(Ix**2 + Iy**2) > 1:
#        beta = degrees(atan(Iy/Ix))
#        alpha = 0
#        if Ix < 0 and Iy >= 0:
#            beta = abs(beta)
#        elif Ix > 0 and Iy >= 0:
#            beta = 180-abs(beta)
#        elif Ix > 0 and Iy <= 0:
#            beta = 180+abs(beta)
#        elif Ix < 0 and Iy <= 0:
#            beta = 360-abs(beta)
#
#    elif Ix == 0 and sqrt(Ix**2 + Iy**2) > 1:
#        alpha = 0
#        if Iy > 0:
#            beta = 0
#        elif Iy < 0:
#            beta = 0
#
#    if alpha > 35:
#        alpha = 35

#    alpha -= alpha / 8
#    alpha += (-Ix*30) / 8
#    beta -= beta / 8
#    beta += (-Iy*30) / 8
    
    #alpha = -Ix *(1 - Ix*0.1) * 35.0
    #beta = -Iy * (1 - Iy*0.1) * 35.0
    #
    # IMPORTANT:
    # - tuning of these values might be necessary
    # - depending on motor polarity a - might be required
    #    e.g. alpha,beta = - ....
    # - depending on motor connections X and Y might be interchanged
    #
    alpha = IxSgn * Ix * IxToAlpha
    beta = IySgn * Iy *  IyToAlpha
    if IswapXY :
        tmp=alpha
        alpha=beta
        beta=alpha

#    alpha =  -Ix * IxToAlpha
#    beta = Iy *  IyToAlpha

    if alpha < -15:
        alpha = -15
    elif alpha>15:
        alpha = 15
    if beta < -15:
        beta = -15
    elif beta>15:
        beta = 15

    alpha = prevAlpha * omega + (1-omega) * alpha
    beta = prevBeta * omega + (1-omega) * beta

    alpha = round(round(alpha / 0.2) * 0.2, -int(floor(log10(0.2))))   ## permet d'arrondire avec 0.2 de précision
    beta = round(round(beta / 0.2) * 0.2, -int(floor(log10(0.2))))

#    alpha = round(round(alpha / 0.5) * 0.5, -int(floor(log10(0.5))))   ## permet d'arrondire avec 0.2 de précision
#    beta = round(round(beta / 0.5) * 0.5, -int(floor(log10(0.5))))
#    if alpha <= 35 and beta <= 360 and arduinoIsConnected == True and startBalanceBall == True:
#    if alpha <= 35 and beta <= 360 and arduinoIsConnected == True and startBalanceBall == True:
#        ser.write((str(dataDict[(alpha,beta)])+"\n").encode())
    if startBalanceBall:
       ser.write((str(alpha) + "," + str(beta)+"\n").encode())

    #print(Ix,Iy,ballPosX,ballPosY,"alpha=",alpha,"beta=",beta,dataDict[(alpha,beta)],prevBallPosX,prevBallPosY,sommeErreurX,sommeErreurY)
#    print(Ix*10000,Iy*10000,ballPosX,ballPosY,"alpha=",alpha,"beta=",beta,prevBallPosX,prevBallPosY,sommeErreurX,sommeErreurY)

#    if startBalanceBall == True:
    sommeErreurX += (consigneX-ballPosX)
    sommeErreurY += (consigneY-ballPosY)


prevX,prevY = 0,0
x,y = 0,0
prevConsigneX, prevConsigneY = 0,0
start_time = 0
def main():
    start_timeFPS = time.time()
    global H,S,V
    global getPixelColor
    global x,y, alpha, beta
    global prevX, prevY, prevAlpha, prevBeta, prevConsigneX, prevConsigneY
    global consigneX, consigneY, sommeErreurX, sommeErreurY
    global camWidth,camHeight
    global timeInterval, start_time
    global showVideoWindow
    
    _, img=cam.read()
    img = img[0:int(camHeight),int((camWidth-camHeight)/2):int(camWidth-((camWidth-camHeight)/2))] #[Y1:Y2,X1:X2]
    imgCircle = np.zeros(img.shape, dtype=np.uint8)
    cv2.circle(imgCircle, (240,240), 270, (255, 255, 255), -1, 8, 0)
    img = img & imgCircle
    imgHSV = cv2.cvtColor(img,cv2.COLOR_BGR2HSV)
    
    if getPixelColor == True and mouseY > 0 and mouseY < 480 and mouseX < 480:
        pixelColorOnClick = img[mouseY,mouseX]
        pixelColorOnClick = np.uint8([[pixelColorOnClick]])
        pixelColorOnClick = cv2.cvtColor(pixelColorOnClick,cv2.COLOR_BGR2HSV)
        H = pixelColorOnClick[0,0,0]
        S = pixelColorOnClick[0,0,1]
        V = pixelColorOnClick[0,0,2]
        #print(mouseX, mouseY)
        #print(pixelColorOnClick)
        getPixelColor = False
    
    lowerBound=np.array([H-sliderH.get(),S-sliderS.get(),V-sliderV.get()])
    upperBound=np.array([H+sliderH.get(),S+sliderS.get(),V+sliderV.get()])

    mask=cv2.inRange(imgHSV,lowerBound,upperBound)
    mask = cv2.blur(mask,(6,6))                        # ajoute du flou à l'image

    ##    mask = np.uint8(mask) 

    #thresh=sliderS.get()
    #if thresh>0:
    #    thresh = thresh*255/100
    #thresh=200
    #imBW=cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
    #imBW=cv2.GaussianBlur(imBW,(11,11),0)
    #imBW=cv2.threshold(imBW,thresh,255,cv2.THRESH_BINARY)[1]
##
    #lo=sliderH.get()
    #if lo>0:
    #    lo = lo*255/100
    #hi=sliderV.get()
    #if hi>0:
    #    hi = hi*255/100
    #mask=cv2.inRange(imBW,lo,hi)


    mask = cv2.erode(mask, None, iterations=2)         # retire les parasites
    mask = cv2.dilate(mask, None, iterations=2)        # retire les parasites


    cnts = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)
    #print("Imutils:", imutils.is_cv2())
    #cnts = imutils.grab_contours(cnts)
#    if cnts != None and len(cnts)>0:
       #cnts = cnts[0] if imutils.is_cv2() else cnts[1]
    cnts = cnts[0]
    center = None


    cv2.circle(img, (int(consigneX), int(consigneY)), int(4),(0, 255, 0), 2)
    if showCalqueCalibrationBool == True:
        cv2.circle(img, (240,240), 220,(255, 0, 0), 2)
        cv2.circle(img, (240,240), 160,(255, 0, 0), 2)
        cv2.line(img, (240, 240), (240, 240+160), (255,0,0), 2) # Axis 0
        cv2.line(img, (240, 240), (240+138, 240-80), (255,0,0), 2) # Axis 1
        cv2.line(img, (240, 240), (240-138, 240-80), (255,0,0), 2) # Axis 2
    if len(cnts) > 0:
        c = max(cnts, key=cv2.contourArea)
        timeInterval = time.time() - start_time
        (x, y), radius = cv2.minEnclosingCircle(c)
        
        if radius > 10:
            cv2.putText(img,str(int(x)) + ";" + str(int(y)).format(0, 0),(int(x)-50, int(y)-50), cv2.FONT_HERSHEY_SIMPLEX,1, (255, 255, 255), 2)
            cv2.circle(img, (int(x), int(y)), int(radius),(0, 255, 255), 2)
            #if startBalanceBall:
            PIDcontrol(int(x),int(y),prevX,prevY,consigneX,consigneY)
            start_time = time.time()
    else:
        sommeErreurX, sommeErreurY = 0,0


    if showVideoWindow == True:
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        img = Image.fromarray(img)
        imgtk = ImageTk.PhotoImage(image=img)
        lmain.imgtk = imgtk
        lmain.configure(image=imgtk)
    lmain.after(5, main)

    drawWithBall()
    if prevConsigneX != consigneX or prevConsigneY != consigneY:
        sommeErreurX, sommeErreurY = 0,0

    paintGraph()
    prevX,prevY = int(x), int(y)
    prevConsigneX, prevConsigneY = consigneX, consigneY
    prevAlpha = alpha
    prevBeta = beta

#    print("FPS: ", 1.0 / (time.time() - start_timeFPS))



#FrameVideoControl = tk.LabelFrame(controllerWindow, text="Vidéo contrôle")
FrameVideoControl = tk.LabelFrame(controllerWindow, text="Vidéo control")
FrameVideoControl.place(x=20,y=20,width=380)
#BRetourVideo = tk.Button(FrameVideoControl, text="Afficher le retour vidéo", command=showCameraFrameWindow)
BRetourVideo = tk.Button(FrameVideoControl, text="Show video feedback", command=showCameraFrameWindow)
BRetourVideo.pack()
BPositionCalibration = tk.Button(FrameVideoControl, text="Calque", command=showCalqueCalibration)
BPositionCalibration.place(x=290,y=0)

sliderH = tk.Scale(FrameVideoControl, from_=0, to=100, orient="horizontal", label="Sensibilité H", length=350, tickinterval = 10)
sliderH.set(sliderHDefault)
sliderH.pack()
sliderS = tk.Scale(FrameVideoControl, from_=0, to=100, orient="horizontal", label="Sensibilité S", length=350, tickinterval = 10)
sliderS.set(sliderSDefault)
sliderS.pack()
sliderV = tk.Scale(FrameVideoControl, from_=0, to=100, orient="horizontal", label="Sensibilité V", length=350, tickinterval = 10)
sliderV.set(sliderVDefault)
sliderV.pack()



#FrameServosControl = tk.LabelFrame(controllerWindow, text="Servos contrôle")
FrameServosControl = tk.LabelFrame(controllerWindow, text="Servo control",height=390,width=380)
FrameServosControl.place(x=20,y=270,width=380)
#BAbaissementPlateau = tk.Button(FrameServosControl, text="Ranger les bras", command=rangerPlateau)
BAbaissementPlateau = tk.Button(FrameServosControl, text="Store the arms", command=rangerPlateau)
#BAbaissementPlateau.pack()
BAbaissementPlateau.place(x=200,y=0)
#BElevationBras = tk.Button(FrameServosControl, text="Mettre en place le plateau", command=eleverPlateau)
BElevationBras = tk.Button(FrameServosControl, text="Setup platform (level)", command=eleverPlateau)
#BElevationBras.pack()
BElevationBras.place(x=000,y=0)
#BTesterServos = tk.Button(FrameServosControl, text="Tester les servomoteurs", command=servosTest)
BTesterServos = tk.Button(FrameServosControl, text="Test Servo motors", command=servosTest)
#BTesterServos.pack()
BTesterServos.place(x=000,y=30)
BTesterServos = tk.Button(FrameServosControl, text="Test Axis 0", command=servoTestAxis_0)

#BTesterServos.pack()
BTesterServos.place(x=000,y=60)

BTesterServos = tk.Button(FrameServosControl, text="Test Axis 1", command=servoTestAxis_1)
#BTesterServos.pack()
BTesterServos.place(x=120,y=60)

BTesterServos = tk.Button(FrameServosControl, text="Test Axis 2", command=servoTestAxis_2)
#BTesterServos.pack()
BTesterServos.place(x=240,y=60)

sliderServoP = tk.Scale(FrameServosControl, from_=0, to=250, orient="horizontal", label="Servo P", length=350, tickinterval = 50, resolution=5)
sliderServoP.set( 150)
sliderServoP.place(x=0,y=90)
def setServoP(event):
    global ser
    p=sliderServoP.get()
    print(p)
    ser.write(("kp{}\n".format(p)).encode())

sliderServoP.bind("<ButtonRelease-1>", setServoP)

sliderServoI = tk.Scale(FrameServosControl, from_=0, to=1, orient="horizontal", label="Servo I", length=350, tickinterval = 0.2, resolution=0.01)
sliderServoI.set( 0.1)
sliderServoI.place(x=0,y=160)
def setServoI(event):
    global ser
    i=sliderServoI.get()
    ser.write(("ki{}\n".format(i)).encode())

sliderServoI.bind("<ButtonRelease-1>", setServoI)

sliderServoD = tk.Scale(FrameServosControl, from_=0, to=5, orient="horizontal", label="Servo D", length=350, tickinterval = 1, resolution=0.1)
sliderServoD.set( 0.3)
sliderServoD.place(x=0,y=230)
def setServoD(event):
    global ser
    d=sliderServoD.get()
    ser.write(("kd{}\n".format(d)).encode())

sliderServoD.bind("<ButtonRelease-1>", setServoD)

sliderServoLimit = tk.Scale(FrameServosControl, from_=0, to=255, orient="horizontal", label="Servo Limit", length=350, tickinterval = 50, resolution=5)
sliderServoLimit.set( 225)
sliderServoLimit.place(x=0,y=300)
def setServoLimit(event):
    global ser
    l=sliderServoLimit.get()
    ser.write(("l{}\n".format(l)).encode())


sliderServoLimit.bind("<ButtonRelease-1>", setServoLimit)


FramePIDCoef = tk.LabelFrame(controllerWindow, text="PID coefficients")
FramePIDCoef.place(x=420,y=20,width=380)
#BafficherGraph = tk.Button(FramePIDCoef, text="Afficher graphique", command=showGraphWindow)
BafficherGraph = tk.Button(FramePIDCoef, text="Open graph", command=showGraphWindow)
BafficherGraph.pack()
sliderCoefP = tk.Scale(FramePIDCoef, from_=0, to=15, orient="horizontal", label="P", length=350, tickinterval = 3, resolution=0.01)
sliderCoefP.set(sliderCoefPDefault)
sliderCoefP.pack()
sliderCoefI = tk.Scale(FramePIDCoef, from_=0, to=1, orient="horizontal", label="I", length=350, tickinterval = 0.2, resolution=0.001)
sliderCoefI.set(sliderCoefIDefault)
sliderCoefI.pack()
sliderCoefD = tk.Scale(FramePIDCoef, from_=0, to=10, orient="horizontal", label="D", length=350, tickinterval = 2, resolution=0.01)
sliderCoefD.set(sliderCoefDDefault)
sliderCoefD.pack()


#FrameBallControl = tk.LabelFrame(controllerWindow, text="Bille contrôle")
FrameBallControl = tk.LabelFrame(controllerWindow, text="Ball control")
FrameBallControl.place(x=420,y=270,width=380, height= 132)
#BballDrawCircle = tk.Button(FrameBallControl, text="Faire tourner la bille en cercle", command=startDrawCircle)
BballDrawCircle = tk.Button(FrameBallControl, text="Spin ball in a circle", command=startDrawCircle)
BballDrawCircle.pack()
#BballDrawEight = tk.Button(FrameBallControl, text="Faire tourner la bille en huit", command=startDrawEight)
BballDrawEight = tk.Button(FrameBallControl, text="Spin ball in figure 8", command=startDrawEight)
BballDrawEight.pack()

#BStartBalance = tk.Button(FrameServosControl, text="Démarrer", command=startBalance, highlightbackground = "#36db8b")
BStartBalance = tk.Button(FrameBallControl, text="To Start up", command=startBalance, highlightbackground = "#36db8b")
#BStartBalance.pack()
BStartBalance.pack()



#FrameBallControl = tk.LabelFrame(controllerWindow, text="Bille contrôle")
FramePlatformControl = tk.LabelFrame(controllerWindow, text="Platform control")
FramePlatformControl.place(x=420,y=420,width=380, height= 250)
#BballDrawCircle = tk.Button(FrameBallControl, text="Faire tourner la bille en cercle", command=startDrawCircle)
sliderX= tk.Scale(FramePlatformControl, from_=-20, to=20, orient="horizontal", label="X-Angle", length=350, tickinterval = 5, resolution=0.1)

sliderY= tk.Scale(FramePlatformControl, from_=-20, to=20, orient="horizontal", label="Y-Angle", length=350, tickinterval = 5, resolution=0.1)

sliderY.set(0)
sliderX.set(0)

def setAngles(event):
    global ser
    print(event,sliderX.get(),sliderY.get())
    ser.write(("{},{}\n".format(sliderX.get(),sliderY.get())).encode())

sliderX.bind("<ButtonRelease-1>", setAngles)
sliderY.bind("<ButtonRelease-1>", setAngles)
sliderX.pack()
sliderY.pack()

def setZero():
   global ser
   print("Set Zero")
   ser.write("r\n".encode())
   sliderX.set(0)
   sliderY.set(0)

BSetZero= tk.Button(FramePlatformControl, text="Set Zero", command=setZero)
BSetZero.pack()
varIxSgn = tk.IntVar()
if IxSgn < 0 :
    varIxSgn.set(1)
varIySgn = tk.IntVar()
if IySgn < 0 :
    varIySgn.set(1)
varIswapXY = tk.IntVar()
if IswapXY :
    varIySgn.set(1)

def setIxSgn():
   global IxSgn
   print("Set Ix Sgn",varIxSgn.get())
   if varIxSgn.get() == 1:
       IxSgn = -1
   else:
       IxSgn = 1


InvertX = tk.Checkbutton(FramePlatformControl, text="Invert X", var=varIxSgn, command=setIxSgn)
InvertX.place(x=30,y=160)
def setIySgn():
   global IySgn
   if varIySgn.get() == 1:
       IySgn = -1
   else:
       IySgn = 1
InvertY = tk.Checkbutton(FramePlatformControl, text="Invert Y", var=varIySgn, command=setIySgn)
InvertY.place(x=110,y=160)
def setSwapxy():
   global IswapXY 
   if varIswapXY.get() == 1:
       IswapXY = True  
   else:
       IswapXY = False 
SwapXY = tk.Checkbutton(FramePlatformControl, text="Swap X Y",var = varIswapXY, command =setSwapxy)
SwapXY.place(x=190,y=160)



#label = tk.Label(controllerWindow, text="Arduino déconnecté  ", fg="red", anchor="ne")
label = tk.Label(controllerWindow, text="Arduino disconnected", fg="red", anchor="ne")
label.pack(fill="both")
BReset = tk.Button(controllerWindow, text = "Reset", command = resetSlider)
BReset.place(x=20, y=670)
#BConnect = tk.Button(controllerWindow, text = "Connexion", command = connectArduino, background="black")
BConnect = tk.Button(controllerWindow, text = "Connection", command = connectArduino )
BConnect.place(x=100, y=670)
#BQuit = tk.Button(controllerWindow, text = "Quitter", command = endProgam)
BQuit = tk.Button(controllerWindow, text = "Quit", command = endProgam)
BQuit.place(x=730, y=670)


showGraphPositionX = tk.IntVar()
showGraphPositionX.set(1)
#CheckbuttonPositionX = tk.Checkbutton(graphWindow, text="Position en X", variable=showGraphPositionX, command=refreshGraph)
CheckbuttonPositionX = tk.Checkbutton(graphWindow, text="x-Position", variable=showGraphPositionX, command=refreshGraph)
CheckbuttonPositionX.place(x=500,y=20)
showGraphPositionY = tk.IntVar()
showGraphPositionY.set(1)
#CheckbuttonPositionY = tk.Checkbutton(graphWindow, text="Position en Y", variable=showGraphPositionY, command=refreshGraph)
CheckbuttonPositionY = tk.Checkbutton(graphWindow, text="y-Position", variable=showGraphPositionY, command=refreshGraph)
CheckbuttonPositionY.place(x=500,y=40)
showGraphAlpha = tk.IntVar()
#CheckbuttonAlpha = tk.Checkbutton(graphWindow, text="Inclinaison du plateau", variable=showGraphAlpha, command=refreshGraph)
CheckbuttonAlpha = tk.Checkbutton(graphWindow, text="platform inclination", variable=showGraphAlpha, command=refreshGraph)
CheckbuttonAlpha.place(x=500,y=60)




connectArduino()
main()
tk.mainloop()






